#!/bin/bash
#set -xv
#source init_hadcon.sh

writePortA()
{
    if [ $# -gt 0 ]
    then
	echo "RGWR ${PORTA:?undefined} $1" >${hadcon:?undefined}
	sleep 0.005
    else
	echo "error writePortA: too few arguments" 
    fi
}

readPortA()
{
    echo "RGRE ${PINA:?undefined}" >${hadcon:?undefined}
    sleep 0.005
}


#bit 0: CLK
#bit 1: DOUT
#bit 2: SS
#bit 3: ???

#bit 4: DIN


writeDataHigh()
{
  # clock Low
  writePortA 0
  # clock Low  + data high
  writePortA 2
  # clock High + data high
  writePortA 3
  # clock High + data low
  writePortA 1
  # clock low  + data low
  writePortA 0
}

writeDataLow()
{
  # clock Low
  writePortA 0
  # clock low + data low
  writePortA 0
  # clock High + data low
  writePortA 1
  # clock High + data low
  writePortA 1
  # clock low + data low
  writePortA 0
}

#readBitSequence nBits
readBitSequence()
{
    if [ $# -ge 1  ]
    then
        #make sure we start from clock 0 
        writePortA 0

	local nBits=$(( $1 - 1 ))
        #initial bit read differently !!! TNX^6 Peter Wieczorek ;-)
        # since apfel needs first falling clock edge to activate the output pad
        # therefore first cycle then read data.
        #clock high
          writePortA 1
        #clock low
          writePortA 0
        #read data in
          readPortA          

        local n;
	for ((n=0; n<$nBits; n++))
	do 
        #clock high
          writePortA 1
        #read data in
          readPortA          
        #clock low
          writePortA 0
        #  sleep 0.1
        done
    else
	echo "error readBitSequence: too few arguments" 
    fi
}

#writeClockSequence num
writeClockSequence()
{
    if [ $# -ge 1  ]
    then
	local int=$1
        local n;
	for ((n=0; n<$int; n++))
	do 
          writeDataLow
        done
    else
	echo "error writeClockSequence: too few arguments" 
    fi
}

#clearDataInput
#  sends enough empty clock cycles to clear input buffer
#  after each byte add 3 0 writes
clearDataInput()
{
  local n=$(($nCommandBits + $nValueBits + $nChipIdBits + 2));
  local i;

  for ((i=0;$i<$n;i++))
  do
      if [ $(($i%8)) -eq 0 ] 
      then
         writePortA 0
         writePortA 0
      fi
      writeClockSequence 1

  done
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
  writePortA 0
}

#mandatory header to all command sequences
startStreamHeader()
{
  clearDataInput

  # clock Low
  writePortA 0
  # clock Low  + data high
  writePortA 2
  # clock High + data high
  writePortA 3
  # clock low  + data high
  writePortA 2
  # clock low  + data low
  writePortA 0
}


#writeBitSequence #bits #data #endianess (0: little, 1:big)
writeBitSequence()
{
    if [ $# -ge 3  ]
    then
	local num=$1;
	local data=$2;
        local endianness=$3
    if [ $endianness -eq 0 ]
        then 
          local bit;
	  for ((bit=0; bit<$num; bit++ ));
	  do
            local value=$(( $(($data>>$bit))&0x1 ))

            #echo d:$data n:$num v:$value b:$bit

            if [ $value -eq 0 ]
	    then
		writeDataLow
            else
		writeDataHigh
	    fi
	  done
        else
	  for ((bit=$num-1; bit>=0; bit-- ));
	  do
            value=$(( $(($data>>$bit))&0x1 ))

            #echo d:$data n:$num v:$value b:$bit

            if [ $value -eq 0 ]
	    then
		writeDataLow
            else
		writeDataHigh
	    fi
	  done
        fi
    else
	echo "error writeBitSequence: too few arguments" 
    fi
}

#setDac dacNr[1..4] value[ 0 ... 3FF ] chipID[0 ... FF]
setDac()
{
    if [ $# -ge 3 ]
    then
        startStreamHeader
        # dacNr
        writeBitSequence $nCommandBits $(($commandSetDac + $1)) 1
        # value
        writeBitSequence $nValueBits $2 1
        # chipId
        writeBitSequence $nChipIdBits $3 1
        #3 intermediate clock cycles equiv. 3 writeDataLow
        writeClockSequence 3
    else
	echo "error setDac: too few arguments" 
    fi

} 

#readDac dacNr[1..4] chipID[0 ... FF]
readDac()
{
    if [ $# -ge 2 ]
    then
        startStreamHeader
        # dacNr
        writeBitSequence $nCommandBits $(($commandReadDac + $1)) 1
        # dummy value
        writeBitSequence $nValueBits 0 1
        # chipId
        writeBitSequence $nChipIdBits $2 1
        #sleep 0.5
        readBitSequence $((2 + $nValueBits + 3)) #15 

    else
	echo "error readDac: too few arguments" 
    fi

} 

#autoCalibration chipId[0 ... FF]
autoCalibration()
{
    if [ $# -ge 1 ]
    then
        startStreamHeader
        # command 
        writeBitSequence $nCommandBits $commandAutoCalibration 1
        # dummy value
        writeBitSequence $nValueBits 0 1
        # chipId
        writeBitSequence $nChipIdBits $1 1

        #calibration sequence clocks 4 * 10bit DAC
        echo "GO for a coffee this takes time ~ 7 min"
        local size=$(( (0x1 << ($nValueBits)) - 1  )) #0x3FF
        writeClockSequence $size
        echo ... did $size
        writeClockSequence $size
        echo ... did $size
        writeClockSequence $size
        echo ... did $size
        writeClockSequence $size
        echo ... did $size
    else
	echo "error autoCalibration: too few arguments" 
    fi
} 

#testPulseSequence pulseHeightPattern[0 ... 1F] chipId[0 ... FF] 
testPulseSequence()
{
    if [ $# -ge 2 ]
    then
        startStreamHeader
        # command 
        writeBitSequence $nCommandBits $commandTestPulse 1
        # pulse height * channel
        writeBitSequence $nValueBits $1 1
        # chipId
        writeBitSequence $nChipIdBits $2 1

        writeClockSequence 0x1
    else
	echo "error testPulse: too few arguments" 
    fi
}

#testPulse pulseHeight[0 ... 1F] channel[1 .. 2 ] chipId[0 ... FF] 
testPulse()
{
    if [ $# -ge 3 ]
    then
        local arg=$(( ($1 << (1)) << (5*($2-1) ) ));
        testPulseSequence $arg $3

        testPulseSequence 0 $3
    else
	echo "error testPulse: too few arguments" 
    fi
} 



#setAmplitude channelId[1 ... 2] chipId[0 ... FF]
setAmplitude()
{
    if [ $# -ge 2 ]
    then
        startStreamHeader
        # command for channel 1/2
        writeBitSequence $nCommandBits $((($commandSetAmplitude + 1) -(${1}-1) )) 1
        # dummy value
        writeBitSequence $nValueBits 0 1
        # chipId
        writeBitSequence $nChipIdBits $2 1
   else
	echo "error setAmplitude: too few arguments" 
    fi
} 

#resetAmplitude channelId[1 ... 2] chipId[0 ... FF]
resetAmplitude()
{
    if [ $# -ge 2 ]
    then
        startStreamHeader
        # command for channel 1/2
        writeBitSequence $nCommandBits $(($commandResetAmplitude + ((${1}-1) * 2))) 1
        # dummy value
        writeBitSequence $nValueBits 0 1
        # chipId
        writeBitSequence $nChipIdBits $2 1
   else
	echo "error resetAmplitude: too few arguments" 
    fi
} 
